import { z } from "zod";

const ALL_DIRECTIVES = ["dmg"] as const;
const DirectiveSchema = z.enum(ALL_DIRECTIVES);
type DirectiveName = (typeof ALL_DIRECTIVES)[number];
type DirectiveHandler = (...args: string[]) => string;

const ProsceniumLangDirectives: Record<DirectiveName, DirectiveHandler> = {
  dmg: (amount = "1", type = "physical") => {
    return `<em class='dmg ${type}'>${amount}</em>`;
  },
};

function processCommands(text: string): string {
  const commandPattern = /\{(\w+):([^}]+)\}/g;

  return text.replace(commandPattern, (_match, command, args) => {
    // TODO: Improve error reporting
    const directive = DirectiveSchema.parse(command);
    const handler = ProsceniumLangDirectives[directive];
    const parsedArgs = [];
    let currentArg = "";
    let inQuotes = false;

    for (let i = 0; i < args.length; i++) {
      const char = args[i];
      if (char === '"' && (i === 0 || args[i - 1] !== "\\")) {
        inQuotes = !inQuotes;
      } else if (char === "," && !inQuotes) {
        parsedArgs.push(currentArg.trim().replace(/^"|"$/g, ""));
        currentArg = "";
      } else {
        currentArg += char;
      }
    }
    parsedArgs.push(currentArg.trim().replace(/^"|"$/g, ""));

    return handler(...parsedArgs);
  });
}

export async function processLang(input: string): Promise<string> {
  const lines = input.split("\n");
  const processedLines: string[] = [];
  for (const line of lines) {
    const processed = processCommands(line);
    processedLines.push(processed);
  }

  return processedLines.join("\n");
}
